package com.yihu.hos.system.service;

import com.yihu.hos.common.ActiveMqUtil;
import com.yihu.hos.common.constants.Constants;
import com.yihu.hos.core.file.FileUtil;
import com.yihu.hos.system.dao.FlowClassDao;
import com.yihu.hos.system.dao.intf.IFlowClassDao;
import com.yihu.hos.system.dao.intf.IFlowDao;
import com.yihu.hos.system.model.SystemServiceFlow;
import com.yihu.hos.system.model.SystemServiceFlowClass;
import com.yihu.hos.system.service.intf.IFlowManage;
import com.yihu.hos.web.framework.model.Result;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Map;

/**
 * 系统流程管理业务类
 *
 * @author HZY
 * @vsrsion 1.0
 * Created at 2016/8/19.
 */
@Service("flowManager")
public class FlowManager implements IFlowManage {

    public static final String BEAN_ID = "flowManager";

    @Resource(name = "flowDao")
    private IFlowDao flowDao;

    @Resource(name = FlowClassDao.BEAN_ID)
    private IFlowClassDao flowClassDao;

    @Override
    public Result getFlowList(Map<String, Object> params) throws Exception {
        return flowDao.getFlowList(params);
    }

    @Override
    public SystemServiceFlow getFlowById(Integer id) throws Exception {
        return flowDao.getEntity(SystemServiceFlow.class, id);
    }

    @Transactional
    public Result addFlow(SystemServiceFlow obj) throws Exception {
        flowDao.saveEntity(obj);
        SystemServiceFlowClass flowClass = obj.getFlowClass();
        flowClass.setFlowId(obj.getId());
        flowDao.saveEntity(flowClass);
        //发送消息到ＭＱ对列
        if (Constants.FLOW_TYPE_ROUTE.equals(flowClass.getType())) {
            //route
            ActiveMqUtil.sendMessage(Constants.FLOW_QUEUE_NAME, Constants.ROUTE_DEFINE_ADD);
        } else if (Constants.FLOW_TYPE_PROCESSOR.equals(flowClass.getType())) {
            //processor
            ActiveMqUtil.sendMessage(Constants.FLOW_QUEUE_NAME, Constants.PROCESSOR_ADD);
        } else {
            return Result.error("未选择类别，清查看！");
        }
        return Result.success("保存成功");
    }

    @Transactional
    public Result updateFlow(SystemServiceFlow obj) throws Exception {
        SystemServiceFlow flow = flowDao.getEntity(SystemServiceFlow.class, obj.getId());
        flow.setCode(obj.getCode());
        flow.setName(obj.getName());
        flow.setPath(obj.getPath());
        flow.setChart(obj.getChart());
        flow.setValid(obj.getValid());
        SystemServiceFlowClass newFlowClass = flowClassDao.getFlowClassByFlowId(obj.getId());
        SystemServiceFlowClass flowClass = obj.getFlowClass();
        newFlowClass.setClassName(flowClass.getClassName());
        newFlowClass.setPackageName(flowClass.getPackageName());
        newFlowClass.setClassPath(flowClass.getClassPath());
        //发送消息到ＭＱ对列
        if (Constants.FLOW_TYPE_ROUTE.equals(flowClass.getType())) {
            //route
            ActiveMqUtil.sendMessage(Constants.FLOW_QUEUE_NAME, Constants.ROUTE_DEFINE_CHANGE);
        } else if (Constants.FLOW_TYPE_PROCESSOR.equals(flowClass.getType())) {
            //processor
            ActiveMqUtil.sendMessage(Constants.FLOW_QUEUE_NAME, Constants.PROCESSOR_DATA_CHANGE);
        } else {
            return Result.error("未选择类别，清查看！");
        }
        return Result.success("更新成功");
    }

    @Transactional
    public Result deleteFlow(Integer id) throws Exception {
        SystemServiceFlow flow = flowDao.getEntity(SystemServiceFlow.class, id);
        SystemServiceFlowClass flowClass = flowClassDao.getFlowClassByFlowId(id);
        flowClassDao.deleteEntity(flowClass);
        flowDao.deleteEntity(flow);
        //发送消息到ＭＱ对列
        if (Constants.FLOW_TYPE_ROUTE.equals(flowClass.getType())) {
            //route
            ActiveMqUtil.sendMessage(Constants.FLOW_QUEUE_NAME, Constants.ROUTE_DEFINE_DELETE);
        } else if (Constants.FLOW_TYPE_PROCESSOR.equals(flowClass.getType())) {
            //processor
            ActiveMqUtil.sendMessage(Constants.FLOW_QUEUE_NAME, Constants.PROCESSOR_DATA_DELETE);
        } else {
            return Result.error("未选择类别，清查看！");
        }

        return Result.success("删除成功");

    }

        /* ===================================  flowClass 部分================================================*/


    @Override
    public Result getFlowClassList(Map<String, Object> params) throws Exception {
        return null;
    }

    @Override
    public SystemServiceFlowClass getFlowClassByFlowId(Integer flowId) throws Exception {
        return flowClassDao.getFlowClassByFlowId(flowId);
    }

    @Transactional
    public Result addFlowClass(SystemServiceFlowClass obj) throws Exception {
        flowDao.saveEntity(obj);
        return Result.success("保存成功");
    }

    @Transactional
    public Result updateFlowClass(SystemServiceFlowClass obj) throws Exception {
        return null;
    }

    @Transactional
    public boolean deleteFlowClassByFlowId(Integer flowId) {
        boolean succ = false;
        try {
            SystemServiceFlowClass flowClass = flowClassDao.getFlowClassByFlowId(flowId);
            flowClassDao.deleteEntity(flowClass);
            succ = true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return succ;
    }



    @Override
    public String uploadFile(MultipartFile file, String baseSavePath) {
        String fileName =  file.getOriginalFilename();
        boolean succ = false;
        try {
            succ = FileUtil.writeFile(baseSavePath +"/"+ fileName, file.getBytes(), "utf-8");
            if (succ){
                return  fileName;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
}
